/******************************************************************
* dft.h - Short Document description~
*  
* 		Longer description.. ................
*
* Created by:	isabella, 2011.09.26 - 11:58:03am
* 
******************************************************************/

#ifndef DFT_H
#define DFT_H

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

// Simple macro to adjust frequency to sample rate 
#define		dftFreq(___f, ___sampleRate) ((((M_PI * 2) * (___f)) / ((float)(___sampleRate))))
// Convert from gain to db
#define		dft2Dbf(___gain) (20.0f*log10f(___gain))
// Convert from db to gain
#define		dft2Gainf(___db) (exp10f((___db)/20.0f))

 #ifdef __cplusplus
  extern "C" {
 #endif

typedef signed short int dftSample_t, *dftSamplePtr_t;

// Possible return types
typedef enum {
	DFT_ERROR	= 0,
	DFT_OK		= 1,
} dftRet_t, *dftRetPtr_t;

// used with dftCreate()
typedef enum {
	DFT_LINEAR	= 0,
	DFT_LOG		= 1,
} dftCreateType_t, *dftCreateTypePtr_t;

// Type of dft2freq to do
typedef enum {
	DFT_WINDOWED	= 0,	// Default
	DFT_CONTINUOUS,
	DFT_FAST,
	DFT_ENDOFLIST
} dftType_t, *dft_TypePtr_t;

// A Single 'Partial'
typedef struct _partial_t {
	// Frequency 
	float		freq;
	// Phase
	float		phase;
	// Amplitude
	float		amp;

	// Peak (if amp>peakAmp peakAmp = amp;)
	float		peakAmp;

	// Running A/B parts
	float		aN;
	float		bN;

	// How many samples are represented by the A/B parts
	// used as an offset on next run
	size_t		aBCount;

	
	// Period of this frequency (in samples)
	float		period;
	// Number of samples in sinTbl
	size_t		sinfTblSize;
	// Pre-rendered sine table at given freq
	float		*pSinfTbl;
	// Pre-rendered sine table at given freq
	float		*pCosfTbl;
} partial_t, *partialPtr_t;

// Instance of the dft 
typedef struct _dft_t {
	// Sample Rate 
	int		sampleRate;
	// Sample Window size (number of samples)
	size_t		sampleWindowSize;

	// dft2Freq algo type
	dftType_t	dftType;

	// number of partials
	size_t		count;

	// list of partials
	partial_t	list[];
} dft_t, *dftPtr_t;

// Initialize partial to given values
dftRet_t	dftInitPartial(partialPtr_t partial, int sampleRate, size_t sampleWindowSize, float freq);

// Initalize dft instance. freqA, freqB = first, last frequency of spectrum. freqStep = stride
dftPtr_t	dftCreate(int sampleRate, size_t sampleWindowSize, float freqA, float freqB, float freqStep, dftCreateType_t dftType);

// Free resources 
void		dftDestroy(dftPtr_t pDft);

// Do PCM -> freq conversion
dftRet_t	dft2Freq(dftPtr_t pDft, dftSamplePtr_t sampleBuf);

// Do Freq -> PCM conversion
dftRet_t	dft2PCM(dftPtr_t pDft, dftSamplePtr_t sampleBuf);

// Do PCM -> freq conversion of a single freq band 
dftRet_t	dft2FreqBand(int sampleRate, dftSamplePtr_t sampleBuf, size_t sampleWindowSize, float freq, float *amp, float *phase);

// Quickie converter from type to a string
static inline const char *dftType2Str(dftType_t type) {
	const char *str = NULL;
	switch(type) {
		case 	DFT_WINDOWED:
			str = "WINDOWED";
			break;
		case	DFT_CONTINUOUS:
			str = "CONTINUOUS";
			break;
		case	DFT_FAST:
			str = "FAST";
			break;
		default:
			str = "Huh?! ~_~";
			break;
	};
	return(str);
}

// Formatted printing facilities
#define	dftErr(__fmt, ...)	(fprintf(stderr, "%s(): ERROR - " __fmt, __FUNCTION__, ##__VA_ARGS__))
#define	dftDbg(__fmt, ...)	(fprintf(stdout, "%s(): DEBUG - " __fmt, __FUNCTION__, ##__VA_ARGS__))
#define	dftInfo(__fmt, ...)	(fprintf(stdout, "%s(): INFO  - " __fmt, __FUNCTION__, ##__VA_ARGS__))

 #ifdef __cplusplus
  }
 #endif 

#endif
