/****************************************************************
**
** Qt tutorial 1 - No Y0 here please! srsly
**
****************************************************************/

#include <qapplication.h>
#include <qpushbutton.h>
#include <qscrollarea.h>
#include <qtextedit.h>
#include <qpen.h>
#include <QSlider>
#include <QVBoxLayout>
#include <QVector>
#include <qobject.h>

#include <stdio.h>
#include <math.h>
#include <qmenubar.h>

#include "qt-dft.h"
#include "main.h"
#include "qt-scope.h"
#include "qt-funcgen.h"


// Stupid Slots

void MainWindow::changeType(void) {
	pDft->setDftType((dftType_t)(pDft->getDftType() + 1));
}


void MainWindow::changePeakDecayRate(void) {
	decayTblNdx = (decayTblNdx + 1) % MW_DECAY_ENTRIES;
	pDft->setPeakDecayRate(decayTbl[decayTblNdx]);
}

void	MainWindow::rangeInc(void) {
	pDft->setRange(pDft->getRange()+1);
}

void	MainWindow::rangeDec(void) {
	pDft->setRange(pDft->getRange()-1);
}

void	MainWindow::rangeReset(void) {
		pDft->setRange(95);
}

void	MainWindow::offsetInc(void) {
	pDft->setOffset(pDft->getOffset()+1);
}

void	MainWindow::offsetDec(void) {
	pDft->setOffset(pDft->getOffset()-1);
}

void	MainWindow::offsetReset(void) {
	pDft->setOffset(0);
}



MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent) {

	// Init internal values
	decayTblNdx 	= 0;

	sampleRate 	= 44100;
	sampleCount 	= sampleRate/14;

	// Create Scope and DFT widgets plz
	pDft 		= new DFT(sampleRate, sampleCount, this);
	pScope		= new Scope(this);

	pScope->resize(0, 50);

	// Create/connect menu signals
	dftRangeInc	= new QAction(tr("Range +"), this);
	dftRangeInc->setShortcuts(QKeySequence::MoveToNextLine);
	connect(dftRangeInc, SIGNAL(triggered()), this, SLOT(rangeInc()));

	dftRangeDec	= new QAction(tr("Range -"), this);
	dftRangeDec->setShortcuts(QKeySequence::MoveToPreviousLine);
	connect(dftRangeDec, SIGNAL(triggered()), this, SLOT(rangeDec()));

	dftRangeReset	= new QAction(tr("Range Max"), this);
	dftRangeReset->setShortcuts(QKeySequence::MoveToStartOfDocument);
	connect(dftRangeReset, SIGNAL(triggered()), this, SLOT(rangeReset()));

	// Offset
	dftOffsetInc	= new QAction(tr("Offset + ."), this);
	dftOffsetInc->setShortcuts(QKeySequence::MoveToNextChar);
	connect(dftOffsetInc, SIGNAL(triggered()), this, SLOT(offsetInc()));

	dftOffsetDec	= new QAction(tr("Offset - ,"), this);
	dftOffsetDec->setShortcuts(QKeySequence::MoveToPreviousChar);
	connect(dftOffsetDec, SIGNAL(triggered()), this, SLOT(offsetDec()));

	dftOffsetReset 	= new QAction(tr("Zero Offset"), this);
	dftOffsetReset->setShortcuts(QKeySequence::MoveToStartOfLine);
	connect(dftOffsetReset, SIGNAL(triggered()), this, SLOT(offsetReset()));

	// Change Type
	dftChangeType 	= new QAction(tr("Change Type"), this);
	dftChangeType->setShortcuts(QKeySequence::New);
	connect(dftChangeType, SIGNAL(triggered()), this, SLOT(changeType()));

	// Change Decay
	dftChangeDecay 	= new QAction(tr("Decay Rate"), this);
	dftChangeDecay->setShortcuts(QKeySequence::Paste);
	connect(dftChangeDecay, SIGNAL(triggered()), this, SLOT(changePeakDecayRate()));

	// Gets me a menu 	
	fileMenu = menuBar()->addMenu(tr("&Dft"));
	fileMenu->addAction(dftChangeType);
	fileMenu->addAction(dftChangeDecay);
	fileMenu->addSeparator();
	fileMenu->addAction(dftRangeInc);
	fileMenu->addAction(dftRangeDec);
	fileMenu->addAction(dftRangeReset);
	fileMenu->addSeparator();
	fileMenu->addAction(dftOffsetInc);
	fileMenu->addAction(dftOffsetDec);
	fileMenu->addAction(dftOffsetReset);

	// Put teh dft and scope on screen
	splitter = new QSplitter();
	splitter->insertWidget(0,pDft);
	splitter->insertWidget(1,pScope);
	splitter->setOrientation(Qt::Vertical);

	setCentralWidget(splitter);

	// Allocate bufr rhams~
	sampleBufPtr 	= (dftSamplePtr_t)calloc(this->sampleCount * AUDIO_CH_COUNT, sizeof(dftSample_t));
	sampleBufLPtr	= (dftSamplePtr_t)calloc(this->sampleCount, sizeof(dftSample_t));
	sampleBufRPtr	= (dftSamplePtr_t)calloc(this->sampleCount, sizeof(dftSample_t));
	sampleBufMonoPtr= (dftSamplePtr_t)calloc(this->sampleCount, sizeof(dftSample_t));

	// Create da pulse object
	record = new PulseRecord(sampleBufPtr, sampleCount * AUDIO_CH_COUNT, sampleRate, AUDIO_CH_COUNT);

	// Connect everything together :p
	QObject::connect(record, SIGNAL(dataReady(void)), this, SLOT(dataReady(void)));

	// Add traces to o-scope plz
	pScope->addTrace(1, sampleBufLPtr, sampleCount, QColor(0xff, 0xaa, 0xee, 0x80));
	pScope->addTrace(0, sampleBufRPtr, sampleCount, QColor(0x00, 0xff, 0x99, 0x80));

	printf("SampleRate: %d/sec\nSample Buffer Count: %d\n", sampleRate, sampleCount);

	funcDialog = new FunctionWindow(this);
	funcDialog->show();

	// Everything, GoGoGo!~
	record->start();
}

MainWindow::~MainWindow() {
	delete(record);
	free(sampleBufPtr);
	free(sampleBufMonoPtr);
	free(sampleBufLPtr);
	free(sampleBufRPtr);

}

void MainWindow::dataReady(void) {
	// mebbe do this as an #if, preproc should fix anyways…?
	if(AUDIO_CH_COUNT == 2) {
		// De-interleave audio data
		unsigned int ndx;
		for(ndx=0; ndx < sampleCount; ndx++) {
			int L 			= sampleBufPtr[ndx << 1];
			int R 			= sampleBufPtr[(ndx << 1) + 1];
			sampleBufLPtr[ndx] 	= (signed short)L;
			sampleBufRPtr[ndx] 	= (signed short)R;
			sampleBufMonoPtr[ndx] 	= (signed short)((L + R) / 2);
		}
	} else {
		// Do normal copy stuffs
		memcpy(sampleBufMonoPtr, sampleBufPtr, sampleCount * sizeof(dftSample_t));
	}
	pDft->toFreq(sampleBufMonoPtr);

	pScope->dataReady();

}

/*
 ! Function Generator Dialog
*/
FunctionWindow::FunctionWindow(QWidget *parent) : QMainWindow(parent) {
	//FuncGen(QWidget *parent, unsigned int sampleRate)
	funcGenMod = new GenericFuncGen(44100, this);
	setCentralWidget(funcGenMod);
//	resize(400,300);
	setWindowTitle("MyDft Function Generator");
	adjustSize();
}

FunctionWindow::~FunctionWindow() {

};




//main comment
int main( int argc, char **argv ) {

	QApplication a( argc, argv );

	MainWindow *mw 	= new MainWindow();
	
	// Set window params
	a.setApplicationName("blah");
	a.setApplicationVersion("1.1");
	a.setOrganizationName("Org-Name = Blah");
	a.setOrganizationDomain("Org Doman = blea.ch");
	a.setObjectName("Ur Momz");

//	Plot			myPlot;

//	QSplitter *splitter = new QSplitter();
//	QTextEdit *textedit = new QTextEdit();
//	QObject::connect(textedit, SIGNAL(textChanged()), mml, SLOT(mmlChanged()));
//	splitter->insertWidget(1,textedit);
//	splitter->insertWidget(0,&roll);
//	splitter->setOrientation(Qt::Vertical);

//	QScrollArea *scrollArea;
//	scrollArea = new QScrollArea(mw);
//	scrollArea->setBackgroundRole(QPalette::Dark);

//	FuncGen		myFuncGen;

//	scrollArea->setWidget(&myPlot);
//	scrollArea->setWidget(&myDFT);

//	scrollArea->setMinimumSize(1200, 325);
	mw->show();
	mw->resize(500,300);

	return a.exec();
}


