/******************************************************************
* qt-dft.cpp - lala
*  
* 		Longer description.. ................
*
* Created by:	isabella, 2011.09.26 - 11:58:09am
* 
******************************************************************/
#include <qwidget.h>
#include <qvector.h>
#include <qobject.h>
#include <qpainter.h>

#include <QDebug>

#include <stdio.h>
#include <math.h>

#include <qt-dft.h>

#include <stdlib.h>

/********************
 ! DFT Plotter
*********************/
DFT::DFT(int sampleRate, size_t sampleCount, QWidget *parent) : QWidget(parent) {
	this->setBackgroundRole(QPalette::Window);
	this->setAutoFillBackground(true);

	this->setMinimumSize(300, 100);
	this->resize(500,250);

	this->sampleRate 	= sampleRate;
	this->sampleCount 	= sampleCount;

	pBgImage 		= NULL;

	maxRange		= 96;	//mod this later
//	maxRange		= 86;	//mod this later
//	offset			= 10;
	offset			= 0;

	peakDecayRate		= 1.14;


	// Generate phase color table 
	{
		int ndx;
		printf("Generating Color Tables!~ ooOOoooO~~~ Ahhh~~\n");
		for(ndx=0; ndx < DFT_RGB_TBL_SIZE; ndx++) {
			int r,g;
			if(ndx< (DFT_RGB_TBL_SIZE/2)) {
				r = 0xff - ndx*2;
			} else {
				r = ndx*2 - 0xff;
			}
			g = 0xff - r;

			rgbTbl[ndx].r = ((float)r * 0.85f);
			rgbTbl[ndx].g = ((float)g * 0.9f);
			rgbTbl[ndx].b = 0;
		}
	}

	// dftCreate(int sampleRate, size_t sampleWindowSize, float freqA, float freqB, float freqStep, dftCreateType_t dftType)
	// Create and initalize the low level dft instance
	pDft = dftCreate(sampleRate, sampleCount, 12.0f, sampleRate/2, 1.125f, DFT_LOG);
}

void DFT::toFreq(dftSamplePtr_t sampleBuf) {
	// Check if we're visible b4 calculating freq
	QRect visibleRect = visibleRegion().boundingRect();	

	// Convert to frequency spectrum
	dft2Freq(pDft, sampleBuf);

	if(visibleRect.isNull() || (visibleRect.width() == 0) || (visibleRect.height() == 0)) {
		return;
	}

	// Redraw graph
	update();	
}

DFT::~DFT() {
	if(pDft) {
		dftDestroy(pDft);
	}
}

int	DFT::setRange(int newRange) {
	if((newRange > 0) && (newRange <= 96)) {
		maxRange = newRange;
	}
	resizeEvent(NULL);
	return(maxRange);
}

int	DFT::getRange(void) {
	return(maxRange);
}

int	DFT::setOffset(int newOffset) {
	if((newOffset >= 0) && (newOffset <= 96)) {
		offset = newOffset;
	}
	resizeEvent(NULL);
	return(offset);
}

int	DFT::getOffset(void) {
	return(offset);
}

dftType_t DFT::setDftType(dftType_t dftType) {
	pDft->dftType = (dftType_t)((int)dftType % DFT_ENDOFLIST);
	resizeEvent(NULL);
	return(pDft->dftType);
}

dftType_t DFT::getDftType(void) {
	return(pDft->dftType);
}


float	DFT::setPeakDecayRate(float rate) {
	peakDecayRate = rate;
	resizeEvent(NULL);
	return(peakDecayRate);
}

float	DFT::getPeakDecayRate(void) {
	return(peakDecayRate);
}

#define DFT_calcY(height, value, offset, range) ((height)-1 - (int)((float)((value) - (offset)) * ((float)(height)/(float)((range)-(offset)))))

#define DFT_TEXT_HEIGHT		14
#define DFT_LEFT_TEXT_MARGIN	40
#define DFT_TEXT_VSPACING	2
#define	DFT_TOP_TEXT_MARGIN	0
#define	DFT_BOTTOM_TEXT_MARGIN	((DFT_TEXT_HEIGHT) * 2 + (DFT_TEXT_VSPACING) * 2)
#define DFT_FREQ_TEXT_WIDTH	60

void DFT::resizeEvent(QResizeEvent * event) {
	// suppress warning
	Q_UNUSED(event) 

	char cBuf[0x100];
	int c;

	// Calc co-effs
	graphHeight = height() - DFT_TOP_TEXT_MARGIN - DFT_BOTTOM_TEXT_MARGIN;

	delete(pBgImage);
	// Recreate static images for new size
	pBgImage = new QImage(width(), height(), QImage::Format_ARGB32);
	pBgImage->fill(qRgb(0,0,0));

	painter.begin(pBgImage);

	// Draw dB lines and texts
	for(c = offset; c < maxRange; c+=10) {
//		int  y = height()-1 - (int)((float)(c - offset) * ((float)height()/(float)(maxRange - offset)));
		int  y = DFT_calcY(graphHeight, c, offset, maxRange) + DFT_TOP_TEXT_MARGIN + 1;

		snprintf(cBuf, sizeof(cBuf), "%3ddB ", 96 - c);

		painter.setPen(QColor(0xaf, 0xaf, 0xaf));
		painter.drawLine(40, y, width(), y);

		painter.setPen(QColor(0xff, 0xff, 0xff));
		painter.drawText(0, y - 7, 100, DFT_TEXT_HEIGHT, Qt::AlignVCenter | Qt::TextSingleLine, QString(cBuf));


	}

	// Draw frequency text
	int xOffset = DFT_LEFT_TEXT_MARGIN + 1;
	int useWidth = width() - xOffset;
	float spacing = (float)useWidth/(float)pDft->count;
	float xPos = (float)xOffset;
//	int barWidth = (spacing>3.0)?(int)spacing-2:1;
	int step = (pDft->count / (useWidth / DFT_FREQ_TEXT_WIDTH));

	painter.setPen(QColor(0xff, 0xff, 0xff));
	for(c=0; c < (int)pDft->count; c++) {
		//if(((c % step) == 0) || (c == (pDft->count-1))) {
		if(((c % step) == 0)) {
			if(pDft->list[c].freq > 1000) {
				snprintf(cBuf, sizeof(cBuf), "%0.02fk", pDft->list[c].freq/1000.0f);
			} else if(pDft->list[c].freq > 100) {
				snprintf(cBuf, sizeof(cBuf), "%0.01f", pDft->list[c].freq);
			} else {
				snprintf(cBuf, sizeof(cBuf), "%0.02f", pDft->list[c].freq);
			}
			painter.drawText(xPos - (DFT_FREQ_TEXT_WIDTH/2) + ((int)spacing/2), height() - DFT_BOTTOM_TEXT_MARGIN + DFT_TEXT_VSPACING * 2, DFT_FREQ_TEXT_WIDTH, DFT_TEXT_HEIGHT, Qt::AlignHCenter | Qt::AlignBottom | Qt::TextWrapAnywhere, QString(cBuf));			
		}
		xPos += spacing;	//subpixel spacing :D
	}

	// Draw status texts 
	snprintf(cBuf, sizeof(cBuf), " %dHz:%dCH    %d~ %ddB    Dft:%s     Fps:%d   Decay:%0.02f", sampleRate, AUDIO_CH_COUNT, 96-maxRange, 96-offset, dftType2Str(pDft->dftType), sampleRate/sampleCount, peakDecayRate);

//	painter.setPen(QColor(0xff, 0xa0, 0xe0));
	painter.drawText(0, height() - DFT_TEXT_HEIGHT + 1, width(), DFT_TEXT_HEIGHT, Qt::AlignTop | Qt::AlignCenter | Qt::TextSingleLine, QString(cBuf));

	painter.end();

}

void DFT::paintEvent(QPaintEvent *event) {
	// suppress warning
	Q_UNUSED(event)

	QRect visibleRect = visibleRegion().boundingRect();
	if(!visibleRect.isNull() && (visibleRect.height() != this->height())) {
		resize(this->width(), visibleRect.height());
	}

	painter.begin(this);

	painter.setBackgroundMode(Qt::OpaqueMode);

	int xOffset = DFT_LEFT_TEXT_MARGIN + 1;
	int useWidth = visibleRect.width() - xOffset;
	float spacing = (float)useWidth/(float)pDft->count;
	float xPos = (float)xOffset;
	int barWidth = (spacing>3.0)?(int)spacing-2:1;

	if(pBgImage) {
		painter.drawImage(QPoint(0, 0), *pBgImage);
	}

	if(pDft) {
		QColor graphColor(0x00, 0xef, 0x00);
		unsigned int x;
		unsigned int xHoldoff = 20;
		for(x=0; x < pDft->count; x++) {
			// Draw graph lines

//			float amp = dftGetAmp(pDft, x);	//pDft->list[x].amp;
			float 	amp = pDft->list[x].amp;

			int     y  = DFT_calcY(graphHeight, dft2Dbf(amp + 1), offset, maxRange);
			int     pY = DFT_calcY(graphHeight, dft2Dbf(pDft->list[x].peakAmp + 1), offset, maxRange);

			// ((int)amp / (32768/3/this->height()));
//			int 	cPhase = (int)(pDft->list[x].phase * ((double)0x80 / M_PI)) + 0x80;
			unsigned int 	cPhase = (unsigned int)dftScale(pDft->list[x].phase, M_PI_2, 0x80) + 0x80;
//			int 	cPhase = (int)((float)x * ((float)DFT_RGB_TBL_SIZE / (float)pDft->count));

			dftRgbPtr_t pRgb = &rgbTbl[cPhase];

//			if(amp >= dft2Gainf((float)offset)) {
			if(y < graphHeight) {
				painter.setPen(graphColor);
				painter.drawRect((int)xPos, y + DFT_TOP_TEXT_MARGIN, barWidth, graphHeight - y);
				painter.fillRect((int)xPos, y + DFT_TOP_TEXT_MARGIN, barWidth, graphHeight - y, QColor(pRgb->r, pRgb->g, pRgb->b, 0x9f));
			}

			if((y > pY) && (pY < graphHeight)) {
				painter.setPen(QColor(0xef, 0x00, 0x00));
				painter.drawLine((int)xPos, pY + DFT_TOP_TEXT_MARGIN, (int)xPos + barWidth, pY + DFT_TOP_TEXT_MARGIN);
			}

			// And peak towards 0
			if((pDft->list[x].peakAmp > 0) && (peakDecayRate != 0.0f)) {
				pDft->list[x].peakAmp = pDft->list[x].peakAmp / peakDecayRate;
			}


			if(0 && (x > 0) && (x >= xHoldoff)) { // || ((amp*8 < pDft->list[x-1].amp) && (amp < pDft->list[x+1].amp)) 
				if((amp > pDft->list[x+1].amp) && (dft2Dbf(amp+1) > 30.0f)) {
					char cBuf[0x100];
					snprintf(cBuf, sizeof(cBuf), "%0.01fHz\n%0.01fdb\n%0.03f°", pDft->list[x].freq, dft2Dbf(amp), pDft->list[x].phase);
					painter.setPen(QColor(0xff, 0xff, 0x00));
					painter.drawText((int)(xPos-45), y - 60, 100, 60, Qt::AlignHCenter, QString(cBuf));
					xHoldoff=x+3;
				}
			}
			xPos += spacing;	//subpixel spacing :D
		};
	}
	painter.end();
}

