
#include "qt-pulse.h"

/*
 ! PulseAudio Record
*/
PulseRecord::PulseRecord(dftSamplePtr_t bufr, size_t bufrSize, unsigned int sampleRate, size_t channels) {

	this->bufr 	= bufr;
	this->bufrSize 	= bufrSize;
	this->sampleRate= sampleRate;
	this->channels	= channels;

	ss.format = PA_SAMPLE_S16NE;
	ss.channels = channels;
	ss.rate = sampleRate;

	int		paError;

	s = pa_simple_new(	NULL,			// Use the default server.
				"Isabella's App!~",	// Our application's name.
				PA_STREAM_RECORD,
				NULL,			// Use the default device.
				"Signal Analysis",	// Description of our stream.
				&ss,			// Our sample format.
				NULL,			// Use default channel map.
				NULL,			// Use default buffering attributes.
				&paError		// Error code.
			);

	if(!s) {
		qDebug() << "in: " << __FUNCTION__ << "() - Cannot open default Pulseaudio device for recording :/" ;
		qDebug() << "PulseAudio Error Code [" << paError << "]";
	}
}

PulseRecord::~PulseRecord() {
	pa_simple_free(s);
}

void PulseRecord::run() {
	int 		retVal;
	if(!s) {
		qDebug() << "in: " << __FUNCTION__ << "() - Pulseaudio not available :\\" ;
		return;
	}

	while((retVal = pa_simple_read(s, bufr, bufrSize * sizeof(dftSample_t), NULL)) >= 0) {
		emit dataReady();
//		qDebug() << "in: " << __FUNCTION__ << "() " << sCnt++ << ": emit dataReady(bufr, bufrSize, sampleRate);";
		this->msleep(1);
	};
	
	exec();
}


/*
 ! PulseAudio Playback
*/
PulsePlayback::PulsePlayback(dftSamplePtr_t bufr, size_t bufrSize, unsigned int sampleRate, size_t channels) {

	this->bufr 	= bufr;
	this->bufrSize 	= bufrSize;
	this->sampleRate= sampleRate;
	this->channels	= channels;

	ss.format = PA_SAMPLE_S16NE;
	ss.channels = channels;
	ss.rate = sampleRate;

	int		paError;

	s = pa_simple_new(	NULL,			// Use the default server.
				"Isabella's PB App!~",	// Our application's name.
				PA_STREAM_PLAYBACK,
				NULL,			// Use the default device.
				"Signal Analysis",	// Description of our stream.
				&ss,			// Our sample format.
				NULL,			// Use default channel map.
				NULL,			// Use default buffering attributes.
				&paError		// Error code.
			);

	if(!s) {
		qDebug() << "in: " << __FUNCTION__ << "() - Cannot open default Pulseaudio device for recording :/" ;
		qDebug() << "PulseAudio Error Code [" << paError << "]";
	}
}

PulsePlayback::~PulsePlayback() {
	pa_simple_free(s);
}

int PulsePlayback::write(dftSamplePtr_t pBufr, size_t bufrSize) {
	if(!s) {
		qDebug() << "in: " << __FUNCTION__ << "() - Pulseaudio not available :\\" ;
		return(-1);
	}

	if((pBufr == NULL) || (bufrSize == 0)) {
		pBufr 	 = this->bufr;
		bufrSize = this->bufrSize;
	}

	return(pa_simple_write(s, pBufr, bufrSize * sizeof(dftSample_t), NULL));
}

void PulsePlayback::run() {
	int 		retVal;
	if(!s) {
		qDebug() << "in: " << __FUNCTION__ << "() - Pulseaudio not available :\\" ;
		return;
	}


	pa_simple_flush(s, NULL);
	while((retVal = pa_simple_write(s, bufr, bufrSize * sizeof(dftSample_t), NULL)) >= 0) {
		emit dataRequest();
//		qDebug() << "in: " << __FUNCTION__ << "() " << sCnt++ << ": emit dataReady(bufr, bufrSize, sampleRate);";
		this->msleep(1);
	};
	
	exec();
}


