/******************************************************************
* qt-scope.cpp - Oscillioscope widget 
*  
* 		Longer description.. ................
*
* Created by:	isabella, 2011.11.02 - 06:43:44am
* 
******************************************************************/	

#include "qt-scope.h"

/*
 ! Oscillioscope 
*/
Scope::Scope(QWidget *parent) : QWidget(parent) {
	// Set widget Params
	this->setBackgroundRole(QPalette::Window);
	this->setAutoFillBackground(true);
	this->setMinimumSize(400, 50);
	this->resize(500,100);

	pBgImage 		= NULL;
	pOverlayImage 		= NULL;

	int c;
	for(c=0; c < SCOPE_MAX_TRACES; c++) {
		pTraces[c].pSampleBuf = NULL;
	}
}

void 	Scope::paintEvent(QPaintEvent *event) {
	Q_UNUSED(event) // suppress warning in release builds

	QRect visibleRect = visibleRegion().boundingRect();
	if(!visibleRect.isNull() && (visibleRect.height() != this->height())) {
		resize(this->width(), visibleRect.height());
	}

//	QPainter painter(this);

	painter.begin(this);

	painter.setBackgroundMode(Qt::OpaqueMode);
//	painter.setBackground(QBrush(qRgb(0xff,0,0)));

	if(pBgImage) {
		painter.drawImage(QPoint(0, 0), *pBgImage);
	}

	float yScale = 65536 / visibleRect.height();
	int  yCenter = visibleRect.height() / 2;

	int c;
	for(c=0; c < SCOPE_MAX_TRACES; c++) {
		if(pTraces[c].pSampleBuf != NULL) {
			QPainterPath path;

			dftSamplePtr_t pSampleBuf = pTraces[c].pSampleBuf;
/*
			unsigned int offset = 0;
			while(!((pSampleBuf[offset] <= 0) && (pSampleBuf[offset+1] >= 0))) {
				offset++;
				if(offset >= (pTraces[c].sampleCount / 4)) {
					break;
				}
			};
printf("%d\n", offset);

			float xScale = (pTraces[c].sampleCount - (pTraces[c].sampleCount / 4)) / visibleRect.width();
*/
			float xScale = pTraces[c].sampleCount / visibleRect.width();

			path.moveTo(0, yCenter);
			
			int x;
			for(x=0; x < visibleRect.width(); x+=2) {
				int index = (int)((float)x * xScale);
				int y = pSampleBuf[index] / yScale + yCenter;
				path.lineTo(x, y);
			}

			// Draw Trace
			painter.setPen(pTraces[c].color);
			painter.drawPath(path);
		}
	}
	
	if(pOverlayImage) {
		painter.drawImage(QPoint(0, 0), *pOverlayImage);
	}

	painter.end();
}

void	Scope::resizeEvent(QResizeEvent * event) {
	// suppress warning
	Q_UNUSED(event) 

	delete(pBgImage);
	// Recreate static images for new size
	pBgImage = new QImage(width(), height(), QImage::Format_ARGB32);
	pBgImage->fill(qRgb(0,0,0));
	painter.begin(pBgImage);
	painter.setPen(QColor(0x33, 0x33, 0x33));
	painter.drawLine(0, height()/2, width(), height()/2);
	painter.end();
}

void	Scope::addTrace(unsigned int index, dftSamplePtr_t pSampleBuf, size_t sampleCount, QColor color) {
	if(index < SCOPE_MAX_TRACES) {
		pTraces[index].pSampleBuf 	= pSampleBuf;
		pTraces[index].sampleCount	= sampleCount;
		pTraces[index].color		= color;
	}
}

void	Scope::removeTrace(unsigned int index) {
	if(index < SCOPE_MAX_TRACES) {
		pTraces[index].pSampleBuf 	= NULL;
		pTraces[index].sampleCount	= 0;
	}
}

// Convience signal
void 	Scope::dataReady(void) {
	// Ensure we're visable b4 updating
	QRect visibleRect = visibleRegion().boundingRect();	
	if(visibleRect.isNull() || (visibleRect.width() == 0) || (visibleRect.height() == 0)) {
		return;
	}

	update();
}

